@@ -24,13 +24,13 @@
  *
  * Contributor(s): Federico Pinna and Reitek S.p.A.
  *
- * $Revision: 20385 $
+ * $Revision: 21788 $
  * $Author: rjongbloed $
- * $Date: 2008-06-04 12:40:38 +0200 (mer 04 jun 2008) $
+ * $Date: 2008-12-12 05:42:13 +0000 (Fri, 12 Dec 2008) $
  */
 
-#ifndef _PMAILPROTOCOL
-#define _PMAILPROTOCOL
+#ifndef PTLIB_INETMAIL_H
+#define PTLIB_INETMAIL_H
 
 #ifdef P_USE_PRAGMA
 #pragma interface
@@ -184,7 +184,7 @@ class PSMTPClient : public PSMTP
     PBoolean    sendingData;
 
   private:
-    PBoolean _BeginMessage();
+    bool InternalBeginMessage();
 };
 
 
@@ -1019,7 +1019,7 @@ class PRFC822Channel : public PIndirectChannel
 };
 
 
-#endif  // _PMAILPROTOCOL
+#endif  // PTLIB_INETMAIL_H
 
 
 // End Of File ///////////////////////////////////////////////////////////////
