@@ -26,13 +26,13 @@
  *
  * Contributor(s): ______________________________________.
  *
- * $Revision: 20385 $
+ * $Revision: 21950 $
  * $Author: rjongbloed $
- * $Date: 2008-06-04 12:40:38 +0200 (mer 04 jun 2008) $
+ * $Date: 2009-01-27 02:26:00 +0000 (Tue, 27 Jan 2009) $
  */
 
-#ifndef _PPROCESS
-#define _PPROCESS
+#ifndef PTLIB_PROCESS_H
+#define PTLIB_PROCESS_H
 
 #ifdef P_USE_PRAGMA
 #pragma interface
@@ -53,32 +53,34 @@
  */
 #ifdef P_VXWORKS
 #define PCREATE_PROCESS(cls) \
-  PProcess::PreInitialise(0, NULL, NULL); \
   cls instance; \
-  instance._main();
+  instance.InternalMain();
 #elif defined(P_RTEMS)
 #define PCREATE_PROCESS(cls) \
 extern "C" {\
    void* POSIX_Init( void* argument) \
-     { PProcess::PreInitialise(0, 0, 0); \
+     { \
        static cls instance; \
-       exit( instance._main() ); \
+       exit( instance.InternalMain() ); \
      } \
 }
 #elif defined(_WIN32_WCE)
 #define PCREATE_PROCESS(cls) \
-  int WinMain(HINSTANCE, HINSTANCE, LPWSTR, int) \
-    { cls *pInstance = new cls(); \
-      int terminationValue = pInstance->_main(); \
+  int WinMain(HINSTANCE hInst, HINSTANCE, LPWSTR cmdLine, int) \
+    { \
+      cls *pInstance = new cls(); \
+      pInstance->GetArguments().SetArgs(cmdLine); \
+      int terminationValue = pInstance->InternalMain(hInst); \
       delete pInstance; \
       return terminationValue; \
     }
 #else
 #define PCREATE_PROCESS(cls) \
   int main(int argc, char ** argv, char ** envp) \
-    { PProcess::PreInitialise(argc, argv, envp); \
+    { \
       cls *pInstance = new cls(); \
-      int terminationValue = pInstance->_main(); \
+      pInstance->PreInitialise(argc, argv, envp); \
+      int terminationValue = pInstance->InternalMain(); \
       delete pInstance; \
       return terminationValue; \
     }
@@ -137,17 +139,19 @@ class PTimerList : public PObject
           Start
         } action;
 
-        RequestType(Action _action, PTimer * _timer) : action(_action), timer(_timer), id(timer->GetTimerId()), sync(NULL) { }
+        RequestType(Action act, PTimer * t)
+          : action(act)
+          , timer(t)
+          , id(timer->GetTimerId())
+          , sync(NULL)
+        { }
 
         PTimer * timer;
         PTimer::IDType id;
         PSyncPoint * sync;
     };
 
-    typedef std::queue<RequestType> RequestQueueType;
-    typedef std::vector<RequestType> RequestListType;
-
-    void QueueRequest(RequestType::Action action, PTimer * timer, bool _isSync = true);
+    void QueueRequest(RequestType::Action action, PTimer * timer, bool isSync = true);
 
   private:
     mutable PAtomicInteger timerId; 
@@ -155,7 +159,7 @@ class PTimerList : public PObject
     // map used to store timer information
     PMutex timerListMutex;
     struct TimerInfoType {
-      TimerInfoType(PTimer * _timer) : timer(_timer)  { removed = false; }
+      TimerInfoType(PTimer * t) : timer(t)  { removed = false; }
       PTimer * timer;
       bool removed;
     };
@@ -165,8 +169,9 @@ class PTimerList : public PObject
 
     // queue of timer action requests
     PMutex queueMutex;
+    typedef std::queue<RequestType> RequestQueueType;
     RequestQueueType requestQueue;
-    RequestListType addQueue;
+    RequestQueueType addQueue;
 
     // The last system timer tick value that was used to process timers.
     PTimeInterval lastSample;
@@ -278,6 +283,22 @@ class PProcess : public PThread
       PThread & thread
     );
 
+    /**Callback for when a ^C (SIGINT) or termination request (SIGTERM) is
+       received by process.
+
+       Note this function is called asynchronously and there may be many
+       limitations on what can and cannot be done depending on the underlying
+       operating system. It is recommeneded that this does no more than set
+       flags and return.
+
+       Default behaviour returns false and the process is killed.
+
+       @Return true if the process is to be allowed to continue, false otherwise.
+      */
+    virtual bool OnInterrupt(
+      bool terminating ///< true if process terminating.
+    );
+
     /**Determine if the current processes object instance has been initialised.
        If this returns PTrue it is safe to use the PProcess::Current() function.
        
@@ -537,6 +558,14 @@ class PProcess : public PThread
        Directory for OS configuration files.
      */
     static PDirectory GetOSConfigDir();
+
+    /**Get the version of the PTLib library the process is running on, eg
+       "2.5beta3".
+       
+       @return
+       String for library version.
+     */
+    static PString GetLibVersion();
   //@}
 
     /**Get the list of timers handled by the application. This is an internal
@@ -548,22 +577,22 @@ class PProcess : public PThread
     PTimerList * GetTimerList();
 
     /**Internal initialisation function called directly from
-       #_main()#. The user should never call this function.
+       #InternalMain()#. The user should never call this function.
      */
-    static void PreInitialise(
+    void PreInitialise(
       int argc,     // Number of program arguments.
       char ** argv, // Array of strings for program arguments.
       char ** envp  // Array of string for the system environment
     );
 
     /**Internal shutdown function called directly from the ~PProcess
-       #_main()#. The user should never call this function.
+       #InternalMain()#. The user should never call this function.
      */
     static void PreShutdown();
     static void PostShutdown();
 
     /// Main function for process, called from real main after initialisation
-    virtual int _main(void * arg = NULL);
+    virtual int InternalMain(void * arg = NULL);
 
     /**@name Operating System URL manager functions */
     /**
@@ -592,11 +621,11 @@ class PProcess : public PThread
         HostSystemURLHandlerInfo()
         { }
 
-        HostSystemURLHandlerInfo(const PString & _type)
-          : type(_type)
+        HostSystemURLHandlerInfo(const PString & t)
+          : type(t)
         { }
 
-        static bool RegisterTypes(const PString & _types, bool force = true);
+        static bool RegisterTypes(const PString & types, bool force = true);
 
         void SetIcon(const PString & icon);
         PString GetIcon() const;
@@ -618,15 +647,10 @@ class PProcess : public PThread
     };
   //@}
 
-  private:
+  protected:
     void Construct();
 
   // Member variables
-    static int p_argc;
-    static char ** p_argv;
-    static char ** p_envp;
-    // main arguments
-
     int terminationValue;
     // Application return value
 
@@ -666,7 +690,11 @@ class PProcess : public PThread
     int maxHandles;
     // Maximum number of file handles process can open.
 
+    bool m_library;
 
+    PDictionary<POrdinalKey, PThread> activeThreads;
+    PMutex                            activeThreadMutex;
+    
   friend class PThread;
 
 
@@ -678,6 +706,36 @@ class PProcess : public PThread
 #endif
 };
 
+
+/** Class for a process that is a dynamically loaded library.
+ */
+ class PLibraryProcess : public PProcess
+ {
+  PCLASSINFO(PLibraryProcess, PProcess);
+
+  public:
+  /**@name Construction */
+  //@{
+    /** Create a new process instance.
+     */
+    PLibraryProcess(
+      const char * manuf = "",         ///< Name of manufacturer
+      const char * name = "",          ///< Name of product
+      WORD majorVersion = 1,           ///< Major version number of the product
+      WORD minorVersion = 0,           ///< Minor version number of the product
+      CodeStatus status = ReleaseCode, ///< Development status of the product
+      WORD buildNumber = 1             ///< Build number of the product
+    ) : PProcess(manuf, name, majorVersion, minorVersion, status, buildNumber)
+    {
+      m_library = true;
+    }
+  //@}
+
+    ///< Dummy Main() as libraries do not have one.
+    virtual void Main() { }
+};
+
+
 /*
  *  one instance of this class (or any descendants) will be instantiated
  *  via PGenericFactory<PProessStartup> one "main" has been started, and then
@@ -700,16 +758,18 @@ typedef PFactory<PProcessStartup> PProcessStartupFactory;
 // using an inline definition rather than a #define crashes gcc 2.95. Go figure
 #define P_DEFAULT_TRACE_OPTIONS ( PTrace::Blocks | PTrace::Timestamp | PTrace::Thread | PTrace::FileAndLine )
 
-template <unsigned _level, unsigned _options = P_DEFAULT_TRACE_OPTIONS >
+template <unsigned level, unsigned options = P_DEFAULT_TRACE_OPTIONS >
 class PTraceLevelSetStartup : public PProcessStartup
 {
   public:
     void OnStartup()
-    { PTrace::Initialise(_level, NULL, _options); }
+    { PTrace::Initialise(level, NULL, options); }
 };
 
 #endif // PTRACING
 
-#endif
+
+#endif // PTLIB_PROCESS_H
+
 
 // End Of File ///////////////////////////////////////////////////////////////
