@@ -21,9 +21,9 @@
  *
  * Contributor(s): ______________________________________.
  *
- * $Revision: 21337 $
+ * $Revision: 21341 $
  * $Author: rjongbloed $
- * $Date: 2008-10-15 03:52:04 +0000 (Wed, 15 Oct 2008) $
+ * $Date: 2008-10-15 06:36:09 +0000 (Wed, 15 Oct 2008) $
  */
 
 #ifdef __GNUC__
@@ -2561,7 +2561,7 @@ PBoolean PVXMLChannel::QueueResource(const PURL & url, PINDEX repeat, PINDEX del
     return QueuePlayable("URL", url.AsString(), repeat, delay);
 }
 
-PBoolean PVXMLChannel::QueueData(const PBYTEArray & PTRACE_PARAM(data), PINDEX repeat, PINDEX delay)
+PBoolean PVXMLChannel::QueueData(const PBYTEArray & data, PINDEX repeat, PINDEX delay)
 {
   PTRACE(3, "PVXML\tEnqueueing " << data.GetSize() << " bytes for playing");
   PVXMLPlayableData * item = PFactory<PVXMLPlayable>::CreateInstanceAs<PVXMLPlayableData>("PCM Data");
