@@ -26,9 +26,9 @@
  *
  * Contributor(s): ______________________________________.
  *
- * $Revision: 21804 $
+ * $Revision: 22082 $
  * $Author: rjongbloed $
- * $Date: 2008-12-16 02:25:58 +0000 (Tue, 16 Dec 2008) $
+ * $Date: 2009-02-17 05:31:44 +0000 (Tue, 17 Feb 2009) $
  */
 
 #include <ptlib.h>
@@ -40,38 +40,51 @@
 #include <ptlib/pfactory.h>
 #include <ptlib/pprocess.h>
 #include <ptlib/svcproc.h>
+#include "../../../version.h"
 
 #ifdef _WIN32
 #include <ptlib/msos/ptlib/debstrm.h>
-#if defined(P_VIDEO) && defined(P_VFW_CAPTURE)
-namespace PWLibStupidLinkerHacks {
-  extern int loadVideoForWindowsStuff;
-};
-#endif
 #endif
 
-#if defined(P_MINGW)
 namespace PWLibStupidLinkerHacks {
-  extern int loadFakeVideoStuff;
-#if defined(P_AUDIO)
-  extern int loadWindowsMultimediaStuff;
-#endif
-#if defined(P_VIDEO) && defined(P_VFW_CAPTURE)
-  extern int loadVideoForWindowsStuff;
-#endif
-#if defined(P_VIDEO) && defined(P_DIRECTSHOW) && defined(P_DIRECTX)
-  extern int loadDirectShowStuff;
-#endif
-};
-#endif
-     
-#ifdef P_MACOSX
-namespace PWLibStupidLinkerHacks {
-  extern int loadFakeVideoStuff;
-  extern int loadShmVideoStuff;
-  extern int loadCoreAudioStuff;
-};
-#endif
+
+
+#if defined(P_MINGW) || defined(_WIN32) || defined(P_MACOSX)
+
+#  ifdef P_VIDEO
+     extern int loadFakeVideoStuff;
+#    if defined(P_MINGW) || defined(_WIN32)
+#      ifdef P_VFW_CAPTURE
+         extern int loadVideoForWindowsStuff;
+#      endif
+#      if defined(P_DIRECTSHOW)
+         extern int loadDirectShowStuff;
+#      endif
+#    endif
+#    ifdef P_FFVDEV
+       extern int loadFFVideoFileStuff;
+#    endif
+#    ifdef P_VIDFILE
+       extern int loadVideoFileStuff;
+#    endif
+#  endif 
+
+#  ifdef P_AUDIO
+#    if defined(P_MINGW) || defined(_WIN32)
+       extern int loadWindowsMultimediaStuff;
+#    endif
+#    ifdef P_MACOSX
+       extern int loadCoreAudioStuff;
+#    endif
+#  endif
+
+#endif // defined(P_MINGW) || defined(_WIN32) || defined(P_MACOSX)
+
+}; // namespace PWLibStupidLinkerHacks {
+
+
+static const char * const VersionStatus[PProcess::NumCodeStatuses] = { "alpha", "beta", "." };
+
 
 class PExternalThread : public PThread
 {
@@ -113,6 +126,9 @@ class PSimpleThread : public PThread
 };
 
 
+#define new PNEW
+
+
 #ifndef __NUCLEUS_PLUS__
 static ostream * PErrorStream = &cerr;
 #else
@@ -357,6 +373,7 @@ void PTrace::Initialise(unsigned level, const char * filename, const char * roll
                   << " by " << process.GetManufacturer()
                   << " on " << process.GetOSClass() << ' ' << process.GetOSName()
                   << " (" << process.GetOSVersion() << '-' << process.GetOSHardware()
+                  << ") with PTLib (v" << process.GetLibVersion()
                   << ") at " << PTime().AsString("yyyy/M/d h:mm:ss.uuu")
                   << End;
 #endif
@@ -552,14 +569,8 @@ ostream & PTrace::End(ostream & paramStream)
     // should work pretty universally.
     info.stream->width((threadInfo != NULL ? threadInfo->traceLevel : info.currentLevel) + 1);
   }
-  else {
-#ifdef _WIN32_WCE
-    if (!PIsDescendant(info.stream, PDebugStream) && info.stream != &cerr)
-      *info.stream << "\r\n";
-    else
-#endif
+  else
     *info.stream << '\n';
-  }
   info.stream->flush();
 
   info.Unlock();
@@ -637,11 +648,9 @@ PTrace::Block::~Block()
 void PTrace::Cleanup()
 {
 #if P_HAS_THREADLOCAL_STORAGE
-  {
-    PThreadLocalStorage<PThread::TraceInfo> & key = PTraceInfo::Instance().traceStorageKey;
-    delete key.Get();
-    key.Set(NULL);
-  }
+  PThreadLocalStorage<PThread::TraceInfo> & key = PTraceInfo::Instance().traceStorageKey;
+  delete key.Get();
+  key.Set(NULL);
 #endif
 }
 
@@ -993,6 +1002,8 @@ PArgList::PArgList(const char * theArgStr,
   // get the program arguments
   if (theArgStr != NULL)
     SetArgs(theArgStr);
+  else
+    SetArgs(PStringArray());
 
   // if we got an argument spec - so process them
   if (theArgumentSpec != NULL)
@@ -1119,6 +1130,7 @@ void PArgList::SetArgs(const PStringArray & theArgs)
   parameterIndex.SetSize(argumentArray.GetSize());
   for (PINDEX i = 0; i < argumentArray.GetSize(); i++)
     parameterIndex[i] = i;
+  m_argsParsed = 0;
 }
 
 
@@ -1132,8 +1144,8 @@ PBoolean PArgList::Parse(const char * spec, PBoolean optionsBeforeParams)
 
   // If not in parse all mode, have been parsed before, and had some parameters
   // from last time, then start argument parsing somewhere along instead of start.
-  if (optionsBeforeParams && !optionLetters && parameterIndex.GetSize() > 0)
-    arg = parameterIndex[parameterIndex.GetSize()-1] + 1;
+  if (optionsBeforeParams && !optionLetters && m_argsParsed > 0)
+    arg = m_argsParsed;
 
   // Parse the option specification
   optionLetters = "";
@@ -1182,10 +1194,18 @@ PBoolean PArgList::Parse(const char * spec, PBoolean optionsBeforeParams)
       parameterIndex.SetSize(param+1);
       parameterIndex[param++] = arg;
     }
-    else if (optionsBeforeParams && parameterIndex.GetSize() > 0)
+    else if (optionsBeforeParams && parameterIndex.GetSize() > 0) {
+      m_argsParsed = arg;
       break;
-    else if (argStr == "--") // ALL remaining args are parameters not options
-      hadMinusMinus = PTrue;
+    }
+    else if (argStr == "--") {
+      if (optionsBeforeParams)
+        hadMinusMinus = PTrue; // ALL remaining args are parameters not options
+      else {
+        m_argsParsed = arg+1;
+        break;
+      }
+    }
     else if (argStr[1] == '-')
       ParseOption(optionNames.GetValuesIndex(argStr.Mid(2)), 0, arg, canHaveOptionString);
     else {
@@ -1537,7 +1557,7 @@ PProcess::PProcess(const char * manuf, const char * name,
   cout << "Enter program arguments:\n";
   arguments.ReadFrom(cin);
 
-#else // P_RTEMS
+#endif // P_RTEMS
 
 #ifdef _WIN32
   // Try to get the real image path for this process
@@ -1546,55 +1566,44 @@ PProcess::PProcess(const char * manuf, const char * name,
     executableFile = moduleName;
     executableFile.Replace("\\??\\","");
   }
-
-#if defined(P_VIDEO)  && defined(P_VFW_CAPTURE)
-  PWLibStupidLinkerHacks::loadVideoForWindowsStuff = 1;
 #endif
 
-#endif // _WIN32
+#if defined(P_MINGW) || defined(_WIN32) || defined(P_MACOSX)
+
+#  ifdef P_VIDEO
+     PWLibStupidLinkerHacks::loadFakeVideoStuff = 1;
+#    if defined(P_MINGW) || defined(_WIN32)
+#      ifdef P_VFW_CAPTURE
+         PWLibStupidLinkerHacks::loadVideoForWindowsStuff = 1;
+#      endif
+#      if defined(P_DIRECTSHOW)
+         PWLibStupidLinkerHacks::loadDirectShowStuff  =1;
+#      endif
+#    endif
+#    ifdef P_FFVDEV
+       PWLibStupidLinkerHacks::loadFFVideoFileStuff = 1;
+#    endif
+#    ifdef P_VIDFILE
+       PWLibStupidLinkerHacks::loadVideoFileStuff = 1;
+#    endif
+#  endif 
+
+#  ifdef P_AUDIO
+#    if defined(P_MINGW) || defined(_WIN32)
+       PWLibStupidLinkerHacks::loadWindowsMultimediaStuff = 1;
+#    endif
+#    ifdef P_MACOSX
+       PWLibStupidLinkerHacks::loadCoreAudioStuff = 1;
+#    endif
+#  endif
+
+#endif // defined(P_MINGW) || defined(_WIN32) || defined(P_MACOSX)
 
   if (productName.IsEmpty())
     productName = executableFile.GetTitle().ToLower();
 
-#endif // P_RTEMS
-
   Construct();
 
-#if defined(P_MINGW)
-#ifdef P_VIDEO
-  PWLibStupidLinkerHacks::loadFakeVideoStuff = 1;
-#endif
-#if defined(P_AUDIO)
-  PWLibStupidLinkerHacks::loadWindowsMultimediaStuff = 1;
-#endif
-#if defined(P_VIDEO) && defined(P_VFW_CAPTURE)
-  PWLibStupidLinkerHacks::loadVideoForWindowsStuff = 1;
-#endif
-#if defined(P_VIDEO) && defined(P_DIRECTSHOW) && defined(P_DIRECTX)
-  PWLibStupidLinkerHacks::loadDirectShowStuff = 1;
-#endif
-
-#endif
-
-
-#ifdef P_MACOSX
-
-#ifdef P_VIDEO
-
-  PWLibStupidLinkerHacks::loadFakeVideoStuff = 1;
-  
-#ifdef P_SHM_VIDEO
-  PWLibStupidLinkerHacks::loadShmVideoStuff = 1;
-#endif // P_SHM_VIDEO
-
-#endif
-  
-#ifdef P_AUDIO
-  PWLibStupidLinkerHacks::loadCoreAudioStuff = 1;
-#endif // P_AUDIO
-  
-#endif // P_MACOSX
-
   // create one instance of each class registered in the 
   // PProcessStartup abstract factory
   PProcessStartupList & startups = GetPProcessStartupList();
@@ -1710,10 +1719,14 @@ PTime PProcess::GetStartTime() const
 
 PString PProcess::GetVersion(PBoolean full) const
 {
-  const char * const statusLetter[NumCodeStatuses] =
-    { "alpha", "beta", "." };
   return psprintf(full ? "%u.%u%s%u" : "%u.%u",
-                  majorVersion, minorVersion, statusLetter[status], buildNumber);
+                  majorVersion, minorVersion, VersionStatus[status], buildNumber);
+}
+
+
+PString PProcess::GetLibVersion()
+{
+  return psprintf("%u.%u%s%u", MAJOR_VERSION, MINOR_VERSION, VersionStatus[BUILD_TYPE], BUILD_NUMBER);
 }
 
 
@@ -2416,24 +2429,4 @@ PWriteWaitAndSignal::~PWriteWaitAndSignal()
   mutex.EndWrite();
 }
 
-/////////////////////////////////////////////////////////////////////////////
-
-PSystemLog::PSystemLog(Level level)   ///< only messages at this level or higher will be logged
-  : iostream(cout.rdbuf()) 
-{ 
-  logLevel = level; 
-  buffer.log = this; 
-  init(&buffer); 
-}
-
-PSystemLog::PSystemLog(const PSystemLog & other)
-  : PObject(other), iostream(cout.rdbuf()) 
-{
-}
-
-PSystemLog & PSystemLog::operator=(const PSystemLog &)
-{ 
-  return *this; 
-}
-
 // End Of File ///////////////////////////////////////////////////////////////
