@@ -23,9 +23,9 @@
  *
  * Contributor(s): ______________________________________.
  *
- * $Revision: 20747 $
+ * $Revision: 21516 $
  * $Author: rjongbloed $
- * $Date: 2008-08-28 02:10:45 +0000 (Thu, 28 Aug 2008) $
+ * $Date: 2008-11-13 08:04:38 +0000 (Thu, 13 Nov 2008) $
  */
 
 #ifdef __GNUC__
@@ -42,9 +42,9 @@
 /////////////////////////////////////////////////////////////////////////////
 
 PSafeObject::PSafeObject(PSafeObject * indirectLock)
-    : safeReferenceCount(0)
+  : safeReferenceCount(0)
     , safelyBeingRemoved(PFalse)
-    , safeInUse(indirectLock != NULL ? indirectLock->safeInUse : &safeInUseMutex)
+  , safeInUse(indirectLock != NULL ? indirectLock->safeInUse : &safeInUseMutex)
 {
 }
 
@@ -233,11 +233,22 @@ PSafeCollection::~PSafeCollection()
 {
   deleteObjectsTimer.Stop();
 
-  toBeRemoved.AllowDeleteObjects();
-  toBeRemoved.RemoveAll();
+  RemoveAll();
+
+  /* Delete objects moved to deleted list in RemoveAll(), we don't use
+     DeleteObjectsToBeRemoved() as that will do a garbage collection which might
+     prevent deletion. Need to be a bit more forceful here. */
+  for (PList<PSafeObject>::iterator i = toBeRemoved.begin(); i != toBeRemoved.end(); ++i) {
+    i->GarbageCollection();
+    if (i->SafelyCanBeDeleted())
+      delete &*i;
+    else {
+      // If anything still has a PSafePtr .. "detach" it from the collection so
+      // will be deleted whan that PSafePtr finally goes out of scope.
+      i->safelyBeingRemoved = false;
+    }
+  }
 
-  if (deleteObjects)
-    collection->AllowDeleteObjects();
   delete collection;
 }
 
@@ -300,7 +311,15 @@ void PSafeCollection::SafeRemoveObject(PSafeObject * obj)
     removalMutex.Signal();
   }
 
-  obj->SafeDereference();
+  /* Even though we are marked as not to delete objects, we still need to obey
+     the rule that if there are no references left the object is deleted. If
+     the object is still "owned" by a PSafeCollection that has NOT got
+     deleteObjects false, then SafeDereference returns false so we don't delete
+     is here. If there are no PSafePtr()s or PSafeCollections()'s anywhere we
+     need to delete it.
+     */
+  if (obj->SafeDereference() && !deleteObjects)
+    delete obj;
 }
 
 
